import pandas as pd
from sklearn.datasets import load_diabetes
from sklearn.ensemble import RandomForestRegressor

from yikit.feature_selection import BorutaPy


def test_boruta():
    diabetes = load_diabetes()
    X = pd.DataFrame(diabetes["data"], columns=diabetes["feature_names"])
    y = pd.Series(diabetes["target"])

    (
        BorutaPy(
            RandomForestRegressor(n_jobs=-1, random_state=334),
            random_state=334,
        )
        .fit(X, y)
        .support_
    )



# 正規表現でtextファイルと同じやり方で読み込むのですべて手打ちする．

__version__ = "0.3.16-rc.3"
__license__ = "Apache License 2.0"
__author__ = "yu9824"
__copyright__ = "Copyright © 2021 yu9824"
__author_email__ = "yu.9824.job@gmail.com"
__user_id__ = "yu9824"
__app_name__ = "yikit"
__url__ = "http://github.com/yu9824/yikit/"

# __all__ = [
#     'models',
#     'tools',
#     'feature_selection',
#     ]



import importlib.util
import inspect
import sys
from typing import Any, Generic, Optional, TypeVar

if sys.version_info >= (3, 9):
    from collections.abc import Callable, Iterable, Iterator
else:
    from typing import Callable, Iterable, Iterator
T = TypeVar("T")


def is_installed(package_name: str) -> bool:
    """
    Check whether a given Python package is installed.

    Uses `importlib.util.find_spec` to determine if the specified package
    can be imported.

    Parameters
    ----------
    package_name : str
        The name of the package (e.g., "sklearn").

    Returns
    -------
    bool
        True if the package is installed, False otherwise.
    """
    return bool(importlib.util.find_spec(package_name))


def is_argument(__callable: "Callable[..., Any]", arg_name: str) -> bool:
    """
    Check if a given argument name is present in the callable's signature.

    This function checks whether the specified argument name is part of the
    parameters in the callable's signature. It can be used to verify if a
    function or method accepts a specific argument.

    Parameters
    ----------
    __callable : Callable
        The callable (function or method) whose signature is inspected.
    arg_name : str
        The name of the argument to check for in the callable's signature.

    Returns
    -------
    bool
        True if the argument name is found in the callable's parameters, False otherwise.
    """
    return arg_name in set(inspect.signature(__callable).parameters.keys())


# HACK: when drop python3.8, use `dummy_tqdm(Iterable[T])`
class dummy_tqdm(Iterable, Generic[T]):
    """
    A dummy class that mimics the behavior of 'tqdm' for testing or placeholder purposes.

    This class allows you to use a tqdm-like interface in cases where the
    progress bar functionality is not needed or when testing code without
    depending on the actual `tqdm` library.

    Parameters
    ----------
    __iterable : Iterable[T]
        An iterable object that will be wrapped and returned by the class.

    Methods
    -------
    __iter__() -> Iterator[T]
        Returns an iterator for the provided iterable.
    __getattr__(name: str) -> Callable[..., None]
        Returns a no-operation function for unsupported attributes.
    """

    def __init__(
        self, __iterable: Optional["Iterable[T]"] = None, *args, **kwargs
    ) -> None:
        """
        Initialize the dummy tqdm wrapper.

        Parameters
        ----------
        __iterable : Iterable[T], optional
            An iterable object to wrap. If None, an empty tuple is used.
        *args
            Additional positional arguments (ignored, for tqdm compatibility).
        **kwargs
            Additional keyword arguments (ignored, for tqdm compatibility).
        """
        self.__iterable = __iterable if __iterable else ()

    def __iter__(self) -> "Iterator[T]":
        """
        Return an iterator for the given iterable.

        Returns
        -------
        Iterator[T]
            An iterator for the provided iterable object.
        """
        return iter(self.__iterable)

    def __getattr__(self, name: str) -> "Callable[..., None]":
        """
        Handle unsupported attribute access by returning a no-op function.

        This method allows the class to simulate the behavior of tqdm by
        returning a no-op function for any attribute that is not defined.

        Parameters
        ----------
        name : str
            The name of the attribute being accessed.

        Returns
        -------
        Callable[..., None]
            A no-op function that does nothing.
        """
        return self.__no_operation

    @staticmethod
    def __no_operation(*args, **kwargs) -> None:
        """
        A no-operation function used as a placeholder.

        This function does nothing and is used as a fallback for unsupported
        method calls or attributes.

        Returns
        -------
        None
        """
        return



# PyPI登録流れ参考: https://qiita.com/kinpira/items/0a4e7c78fc5dd28bd695

"""
This is a setup.py script generated by py2applet
Usage:
    python setup.py py2app
"""

from setuptools import setup, find_packages
from shutil import copyfile
import os
import sys
import re

# 再帰回数に引っかかるのでとりあえず大きい数に．
sys.setrecursionlimit(10**9)

# ------------------------ ここを変更 --------------------------------
PACKAGE_NAME = "yikit"  # フォルダの名前も統一
DESCRIPTION = "This is my own tool kit."

# py2app用の変数
SRC = ["main.py"]
DATA_FILES = ["LICENSE"]
PKGS = []
ICON = os.path.join("icon", "{}.icns".format(PACKAGE_NAME))
# --------------------------------------------------------------------

VERSION_PYTHON = "{0}.{1}".format(
    sys.version_info.major, sys.version_info.minor
)

# __init__.pyから読み込む
with open(os.path.join(PACKAGE_NAME, "__init__.py")) as f:
    init_text = f.read()
    VERSION = re.search(
        r"__version__\s*=\s*[\'\"](.+?)[\'\"]", init_text
    ).group(1)
    LICENSE = re.search(
        r"__license__\s*=\s*[\'\"](.+?)[\'\"]", init_text
    ).group(1)
    AUTHOR = re.search(r"__author__\s*=\s*[\'\"](.+?)[\'\"]", init_text).group(
        1
    )
    EMAIL = re.search(
        r"__author_email__\s*=\s*[\'\"](.+?)[\'\"]", init_text
    ).group(1)
    ID = re.search(r"__user_id__\s*=\s*[\'\"](.+?)[\'\"]", init_text).group(1)
    APP_NAME = re.search(
        r"__app_name__\s*=\s*[\'\"](.+?)[\'\"]", init_text
    ).group(1)
    url = re.search(r"__url__\s*=\s*[\'\"](.+?)[\'\"]", init_text).group(1)

assert VERSION
assert LICENSE
assert AUTHOR
assert EMAIL
assert ID
assert APP_NAME
assert url

"""
メモ
/.pyenv/versions/anaconda3-2019.03/envs/pymat/lib/python3.7/site-packages/PyQt5/uic/port_v2/ascii_upper.pyの28行目を書き換えた．
少なくともpython3.7においてstringオブジェクトはmaketrans関数を持っておらず，正しくはstr.maketransである．
python3.8で以前やったときはこのエラーがでなかった．
* 3.8にアップデートしてやってみる．
    → 3.8だとsetupのほうがエラー起きるので3.6でやってみる．
    3.6は起動後に関数が呼び出せないと言われる
setup.pyに関する参考サイト
* https://packaging.python.org/guides/distributing-packages-using-setuptools/#packages
py2app 0.23や0.22では動作確認
"""

if "py2app" in sys.argv:
    alias = "-A" in sys.argv or "--alias" in sys.argv

    # 諸変数・定数の定義
    lib_path = os.path.join(os.environ["CONDA_PREFIX"], "lib")
    fname_libpython = "libpython{}.dylib".format(VERSION_PYTHON)

    # libpython3.7.m.dylibだとエラーになるのであらかじめコピーしておく．
    path_original = os.path.join(
        lib_path, "libpython{}m.dylib".format(VERSION_PYTHON)
    )
    path_converted = os.path.join(lib_path, fname_libpython)
    if os.path.exists(path_original) and not os.path.exists(path_converted):
        copyfile(path_original, path_converted)

    # 諸変数の準備
    dylib_files = [
        os.path.join(lib_path, f)
        for f in os.listdir(lib_path)
        if ".dylib" in f
    ]
    contents_path = os.path.join("dist", "{}.app".format(APP_NAME), "Contents")
    frameworks_path = os.path.join(contents_path, "Frameworks")

    OPTIONS = {
        "argv_emulation": False,
        "packages": PKGS,
        "iconfile": ICON,
        "plist": {
            "PyRuntimeLocations": [
                "@executable_path/../Frameworks/{}".format(fname_libpython),
                os.path.join(lib_path, fname_libpython),
            ],
            "CFBundleName": APP_NAME,
            "CFBundleDisplayName": APP_NAME,
            "CFBundleGetInfoString": DESCRIPTION,
            "CFBundleIdentifier": "com.{0}.osx.{1}".format(ID, APP_NAME),
            "CFBundleVersion": VERSION,
            "CFBundleShortVersionString": VERSION,
            "NSHumanReadableCopyright": "Copyright © 2021-, {}".format(AUTHOR),
        },
        # 'frameworks': dylib_files,
    }

    setup(
        name=APP_NAME,
        app=SRC,
        author=AUTHOR,
        author_email=EMAIL,
        version=VERSION,
        data_files=DATA_FILES,
        options={"py2app": OPTIONS},
        setup_requires=["py2app"],
        url="https://github.com/{0}/{1}".format(ID, APP_NAME),
    )

    # aliasモードじゃないとき．
    # if not alias:
    #     {copyfile(f, os.path.join(frameworks_path, os.path.basename(f))) for f in dylib_files}
else:
    """
    参考: https://python-packaging-user-guide-ja.readthedocs.io/ja/latest/distributing.html#manifest-in
    """
    with open("requirements.txt") as requirements_file:
        install_requirements = requirements_file.read().splitlines()

    try:
        with open("README.md") as f:
            long_description = f.read()
    except IOError:
        long_description = ""

    setup(
        name=APP_NAME,
        version=VERSION,
        description=DESCRIPTION,
        long_description=long_description,
        long_description_content_type="text/markdown",  # long_descriptionの形式を'text/plain', 'text/x-rst', 'text/markdown'のいずれかから指定．
        author=AUTHOR,
        author_email=EMAIL,
        maintainer=AUTHOR,
        maintainer_email=EMAIL,
        install_requires=install_requirements,
        url=url,
        keywords="",  # PyPIでの検索用キーワードをスペース区切りで指定．
        license=LICENSE,
        packages=find_packages(exclude=["example"]),
        classifiers=[
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
            "Programming Language :: Python :: 3.10",
            "Programming Language :: Python :: 3.11",
            "Programming Language :: Python :: 3.12",
        ],  # パッケージ(プロジェクト)の分類．https://pypi.org/classifiers/ に掲載されているものを指定可能．
        extras_require={
            "optional": [
                "colorlog",
                "tqdm",
                "ngboost==0.3.6",
                "lightgbm",
                "keras",
            ],
            "test": ["pytest"],
            "dev": ["build"],
        },
    )
